@@ -1,8 +1,9 @@
 use core::fmt::{self, Debug, Display, Formatter};
-use core::marker::PhantomData;
 
 extern crate alloc;
 use alloc::borrow::Cow;
+use alloc::string::String;
+use alloc::vec::Vec;
 
 use serde::{Deserialize, Serialize};
 
@@ -70,53 +71,73 @@ where
 pub trait Message {
     fn id(&self) -> MessageId;
 
-    fn topic(&self, topic_token: &TopicToken) -> Cow<'_, str>;
+    fn topic(&self) -> Option<Cow<'_, str>>;
 
     fn data(&self) -> Cow<'_, [u8]>;
 
     fn details(&self) -> &Details;
+}
 
-    fn retrieve_topic(&self) -> Option<Cow<'_, str>> {
-        let topic_token = match self.details() {
-            Details::Complete(topic_token) => Some(topic_token),
-            Details::InitialChunk(chunk) => Some(&chunk.topic_token),
-            _ => None,
-        };
+#[derive(Debug, Clone, PartialEq, Eq, Serialize, Deserialize)]
+pub struct MessageImpl {
+    id: MessageId,
+    topic: Option<String>,
+    details: Details,
+    data: Vec<u8>,
+}
 
-        topic_token.map(|topic_token| self.topic(topic_token))
+impl MessageImpl {
+    pub fn new<M>(message: &M) -> Self
+    where
+        M: Message,
+    {
+        Self {
+            id: message.id(),
+            data: message.data().to_vec(),
+            topic: message.topic().map(|topic| topic.into_owned()),
+            details: message.details().clone(),
+        }
     }
 }
 
-#[derive(Debug)]
+impl Message for MessageImpl {
+    fn id(&self) -> MessageId {
+        self.id
+    }
+
+    fn topic(&self) -> Option<Cow<'_, str>> {
+        self.topic
+            .as_ref()
+            .map(|topic| Cow::Borrowed(topic.as_str()))
+    }
+
+    fn data(&self) -> Cow<'_, [u8]> {
+        Cow::Borrowed(&self.data)
+    }
+
+    fn details(&self) -> &Details {
+        &self.details
+    }
+}
+
+#[derive(Debug, Clone, PartialEq, Eq, Serialize, Deserialize)]
 pub enum Details {
-    Complete(TopicToken),
+    Complete,
     InitialChunk(InitialChunkData),
     SubsequentChunk(SubsequentChunkData),
 }
 
-#[derive(Debug)]
+#[derive(Debug, Clone, PartialEq, Eq, Serialize, Deserialize)]
 pub struct InitialChunkData {
-    pub topic_token: TopicToken,
     pub total_data_size: usize,
 }
 
-#[derive(Debug)]
+#[derive(Debug, Clone, PartialEq, Eq, Serialize, Deserialize)]
 pub struct SubsequentChunkData {
     pub current_data_offset: usize,
     pub total_data_size: usize,
 }
 
-#[derive(Debug)]
-pub struct TopicToken(PhantomData<*const ()>);
-
-impl TopicToken {
-    /// # Safety
-    /// This function is marked as unsafe because it is an internal API and is NOT supposed to be called by the user
-    pub unsafe fn new() -> Self {
-        Self(PhantomData)
-    }
-}
-
 pub trait Client: Errors {
     fn subscribe<'a, S>(&'a mut self, topic: S, qos: QoS) -> Result<MessageId, Self::Error>
     where
@@ -211,29 +232,178 @@ where
 }
 
 pub trait Connection: Errors {
-    type Message<'a>: Message
-    where
-        Self: 'a;
+    type Message;
 
-    /// GATs do not (yet) define a standard streaming iterator,
-    /// so we have to put the next() method directly in the Connection trait
-    fn next(&mut self) -> Option<Result<Event<Self::Message<'_>>, Self::Error>>;
+    fn next(&mut self) -> Option<Result<Event<Self::Message>, Self::Error>>;
 }
 
 impl<'b, C> Connection for &'b mut C
 where
     C: Connection,
 {
-    type Message<'a>
-    where
-        Self: 'a,
-    = C::Message<'a>;
+    type Message = C::Message;
 
-    fn next(&mut self) -> Option<Result<Event<Self::Message<'_>>, Self::Error>> {
+    fn next(&mut self) -> Option<Result<Event<Self::Message>, Self::Error>> {
         (*self).next()
     }
 }
 
+#[cfg(feature = "alloc")]
+pub mod utils {
+    use core::mem;
+
+    use alloc::sync::Arc;
+
+    use crate::{
+        errors,
+        mutex::{Condvar, Mutex},
+    };
+
+    use super::Event;
+
+    pub struct ConnStateGuard<CV, S>
+    where
+        CV: Condvar,
+    {
+        pub state: CV::Mutex<Option<S>>,
+        pub state_changed: CV,
+    }
+
+    impl<CV, S> ConnStateGuard<CV, S>
+    where
+        CV: Condvar,
+    {
+        pub fn new(state: S) -> Self {
+            Self {
+                state: CV::Mutex::new(Some(state)),
+                state_changed: CV::new(),
+            }
+        }
+    }
+
+    impl<CV, S> ConnStateGuard<CV, S>
+    where
+        CV: Condvar,
+        S: Default,
+    {
+        pub fn new_default() -> Self {
+            Self::new(Default::default())
+        }
+    }
+
+    impl<CV, S> ConnStateGuard<CV, S>
+    where
+        CV: Condvar,
+    {
+        pub fn close(&self) {
+            let mut state = self.state.lock();
+
+            *state = None;
+            self.state_changed.notify_all();
+        }
+    }
+
+    impl<CV, S> Default for ConnStateGuard<CV, S>
+    where
+        CV: Condvar,
+        S: Default,
+    {
+        fn default() -> Self {
+            Self::new(Default::default())
+        }
+    }
+
+    pub struct ConnState<M, E>(Option<Result<Event<M>, E>>);
+
+    impl<M, E> Default for ConnState<M, E> {
+        fn default() -> Self {
+            Self(Default::default())
+        }
+    }
+
+    pub struct Postbox<CV, M, E>(Arc<ConnStateGuard<CV, ConnState<M, E>>>)
+    where
+        CV: Condvar;
+
+    impl<CV, M, E> Postbox<CV, M, E>
+    where
+        CV: Condvar,
+    {
+        pub fn new(connection_state: Arc<ConnStateGuard<CV, ConnState<M, E>>>) -> Self {
+            Self(connection_state)
+        }
+
+        pub fn post(&mut self, event: Result<Event<M>, E>) {
+            let mut state = self.0.state.lock();
+
+            loop {
+                if let Some(data) = &mut *state {
+                    if data.0.is_some() {
+                        state = self.0.state_changed.wait(state);
+                    } else {
+                        break;
+                    }
+                } else {
+                    return;
+                }
+            }
+
+            *state = Some(ConnState(Some(event)));
+            self.0.state_changed.notify_all();
+        }
+    }
+
+    pub struct Connection<CV, M, E>(Arc<ConnStateGuard<CV, ConnState<M, E>>>)
+    where
+        CV: Condvar;
+
+    impl<CV, M, E> Connection<CV, M, E>
+    where
+        CV: Condvar,
+        E: errors::Error,
+    {
+        pub fn new(connection_state: Arc<ConnStateGuard<CV, ConnState<M, E>>>) -> Self {
+            Self(connection_state)
+        }
+    }
+
+    impl<CV, M, E> errors::Errors for Connection<CV, M, E>
+    where
+        CV: Condvar,
+        E: errors::Error,
+    {
+        type Error = E;
+    }
+
+    impl<CV, M, E> super::Connection for Connection<CV, M, E>
+    where
+        CV: Condvar,
+        E: errors::Error,
+    {
+        type Message = M;
+
+        fn next(&mut self) -> Option<Result<Event<Self::Message>, Self::Error>> {
+            let mut state = self.0.state.lock();
+
+            loop {
+                if let Some(data) = &mut *state {
+                    let pulled = mem::replace(data, ConnState(None));
+
+                    match pulled {
+                        ConnState(Some(event)) => {
+                            self.0.state_changed.notify_all();
+                            return Some(event);
+                        }
+                        ConnState(None) => state = self.0.state_changed.wait(state),
+                    }
+                } else {
+                    return None;
+                }
+            }
+        }
+    }
+}
+
 #[cfg(feature = "experimental")]
 pub mod asyncs {
     use core::future::Future;
@@ -282,15 +452,13 @@ pub mod asyncs {
     /// core.stream.Stream is not stable yet and on top of that it has an Item which is not
     /// parameterizable by lifetime (GATs). Therefore, we have to use a Future instead
     pub trait Connection: Errors {
-        type Message: Message;
+        type Message;
 
-        type NextFuture<'a, F, O>: Future<Output = Option<O>> + Send
+        type NextFuture<'a>: Future<Output = Option<Result<Event<Self::Message>, Self::Error>>>
+            + Send
         where
-            Self: 'a,
-            F: FnOnce(&Result<Event<Self::Message>, Self::Error>) -> O + Unpin + Send;
+            Self: 'a;
 
-        fn next<'a, F, O>(&'a mut self, f: F) -> Self::NextFuture<'a, F, O>
-        where
-            F: FnOnce(&Result<Event<Self::Message>, Self::Error>) -> O + Unpin + Send;
+        fn next(&mut self) -> Self::NextFuture<'_>;
     }
 }
